@@ -1,7 +1,7 @@
 /* mpfr_pow_ui-- compute the power of a floating-point
                                   by a machine integer
 
-Copyright 1999-2018 Free Software Foundation, Inc.
+Copyright 1999-2022 Free Software Foundation, Inc.
 Contributed by the AriC and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
@@ -18,7 +18,7 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
+https://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
 #define MPFR_NEED_LONGLONG_H
@@ -89,7 +89,7 @@ mpfr_pow_ui (mpfr_ptr y, mpfr_srcptr x, unsigned long int n, mpfr_rnd_t rnd)
   /* Augment exponent range */
   MPFR_SAVE_EXPO_MARK (expo);
 
-  /* setup initial precision */
+  /* set up initial precision */
   prec = MPFR_PREC (y) + 3 + GMP_NUMB_BITS
     + MPFR_INT_CEIL_LOG2 (MPFR_PREC (y));
   mpfr_init2 (res, prec);
@@ -108,13 +108,13 @@ mpfr_pow_ui (mpfr_ptr y, mpfr_srcptr x, unsigned long int n, mpfr_rnd_t rnd)
       err = prec - 1 - (mpfr_prec_t) i;
       /* First step: compute square from x */
       MPFR_BLOCK (flags,
-                  inexact = mpfr_mul (res, x, x, MPFR_RNDU);
+                  inexact = mpfr_sqr (res, x, MPFR_RNDU);
                   MPFR_ASSERTD (i >= 2);
                   if (n & (1UL << (i-2)))
                     inexact |= mpfr_mul (res, res, x, rnd1);
                   for (i -= 3; i >= 0 && !MPFR_BLOCK_EXCEP; i--)
                     {
-                      inexact |= mpfr_mul (res, res, res, MPFR_RNDU);
+                      inexact |= mpfr_sqr (res, res, MPFR_RNDU);
                       if (n & (1UL << i))
                         inexact |= mpfr_mul (res, res, x, rnd1);
                     });
@@ -140,7 +140,7 @@ mpfr_pow_ui (mpfr_ptr y, mpfr_srcptr x, unsigned long int n, mpfr_rnd_t rnd)
     {
       mpz_t z;
 
-      /* Internal overflow or underflow. However the approximation error has
+      /* Internal overflow or underflow. However, the approximation error has
        * not been taken into account. So, let's solve this problem by using
        * mpfr_pow_z, which can handle it. This case could be improved in the
        * future, without having to use mpfr_pow_z.
