@@ -1,7 +1,7 @@
 /* Test file for mpfr_grandom
 
-Copyright 2011, 2012, 2013 Free Software Foundation, Inc.
-Contributed by the AriC and Caramel projects, INRIA.
+Copyright 2011-2022 Free Software Foundation, Inc.
+Contributed by the AriC and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
 
@@ -17,12 +17,10 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
+https://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
-#include <stdio.h>
-#include <stdlib.h>
-
+#define _MPFR_NO_DEPRECATED_GRANDOM
 #include "mpfr-test.h"
 
 static void
@@ -34,19 +32,19 @@ test_special (mpfr_prec_t p)
   mpfr_init2 (x, p);
 
   inexact = mpfr_grandom (x, NULL, RANDS, MPFR_RNDN);
-  if ((inexact & 3) == 0)
+  if (((unsigned int) inexact & 3) == 0)
     {
       printf ("Error: mpfr_grandom() returns a zero ternary value.\n");
       exit (1);
     }
-  if ((inexact & (3 << 2)) != 0)
+  if (((unsigned int) inexact & (3 << 2)) != 0)
     {
       printf ("Error: the second ternary value of mpfr_grandom(x, NULL, ...)"
               " must be 0.\n");
       exit (1);
     }
 
-  mpfr_clear(x);
+  mpfr_clear (x);
 }
 
 
@@ -55,16 +53,10 @@ test_grandom (long nbtests, mpfr_prec_t prec, mpfr_rnd_t rnd,
               int verbose)
 {
   mpfr_t *t;
-  mpfr_t av, va, tmp;
   int i, inexact;
 
   nbtests = (nbtests & 1) ? (nbtests + 1) : nbtests;
-  t = (mpfr_t *) malloc (nbtests * sizeof (mpfr_t));
-  if (t == NULL)
-    {
-      fprintf (stderr, "tgrandom: can't allocate memory in test_grandom\n");
-      exit (1);
-    }
+  t = (mpfr_t *) tests_allocate (nbtests * sizeof (mpfr_t));
 
   for (i = 0; i < nbtests; ++i)
     mpfr_init2 (t[i], prec);
@@ -72,7 +64,8 @@ test_grandom (long nbtests, mpfr_prec_t prec, mpfr_rnd_t rnd,
   for (i = 0; i < nbtests; i += 2)
     {
       inexact = mpfr_grandom (t[i], t[i + 1], RANDS, MPFR_RNDN);
-      if ((inexact & 3) == 0 || (inexact & (3 << 2)) == 0)
+      if (((unsigned int) inexact & 3) == 0 ||
+          ((unsigned int) inexact & (3 << 2)) == 0)
         {
           /* one call in the loop pretended to return an exact number! */
           printf ("Error: mpfr_grandom() returns a zero ternary value.\n");
@@ -80,9 +73,11 @@ test_grandom (long nbtests, mpfr_prec_t prec, mpfr_rnd_t rnd,
         }
     }
 
-#ifdef HAVE_STDARG
+#if defined(HAVE_STDARG) && !defined(MPFR_USE_MINI_GMP)
   if (verbose)
     {
+      mpfr_t av, va, tmp;
+
       mpfr_init2 (av, prec);
       mpfr_init2 (va, prec);
       mpfr_init2 (tmp, prec);
@@ -109,7 +104,7 @@ test_grandom (long nbtests, mpfr_prec_t prec, mpfr_rnd_t rnd,
 
   for (i = 0; i < nbtests; ++i)
     mpfr_clear (t[i]);
-  free (t);
+  tests_free (t, nbtests * sizeof (mpfr_t));
   return;
 }
 
@@ -119,6 +114,8 @@ main (int argc, char *argv[])
 {
   long nbtests;
   int verbose;
+  int i;
+
   tests_start_mpfr ();
 
   verbose = 0;
@@ -134,6 +131,9 @@ main (int argc, char *argv[])
   test_grandom (nbtests, 420, MPFR_RNDN, verbose);
   test_special (2);
   test_special (42000);
+  /* the following should exercise the case "Extend by 32 bits" in grandom.c */
+  for (i = 0; i < 10000; i++)
+    test_special (1);
 
   tests_end_mpfr ();
   return 0;
