@@ -1,6 +1,6 @@
 /* Test file for mpfr_log.
 
-Copyright 1999, 2001-2019 Free Software Foundation, Inc.
+Copyright 1999, 2001-2022 Free Software Foundation, Inc.
 Contributed by the AriC and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
@@ -83,7 +83,7 @@ check3 (char *s, unsigned long prec, mpfr_rnd_t rnd)
 }
 
 /* examples from Jean-Michel Muller and Vincent Lefevre
-   Cf http://www.ens-lyon.fr/~jmmuller/Intro-to-TMD.htm
+   Cf http://perso.ens-lyon.fr/jean-michel.muller/Intro-to-TMD.htm
 */
 
 static void
