@@ -1,7 +1,7 @@
 /* Test file for mpfr_set_q.
 
-Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012 Free Software Foundation, Inc.
-Contributed by the AriC and Caramel projects, INRIA.
+Copyright 2000-2022 Free Software Foundation, Inc.
+Contributed by the AriC and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
 
@@ -17,35 +17,37 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
+https://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
-#include <stdio.h>
-#include <stdlib.h>
-
 #include "mpfr-test.h"
 
+#ifndef MPFR_USE_MINI_GMP
+
 static void
 check (long int n, long int d, mpfr_rnd_t rnd, const char *ys)
 {
   mpq_t q;
   mpfr_t x, t;
   int inexact, compare;
+  mpfr_flags_t flags, ex_flags;
 
   mpfr_init2 (x, 53);
   mpfr_init2 (t, mpfr_get_prec (x) + mp_bits_per_limb);
   mpq_init (q);
   mpq_set_si (q, n, d);
+  mpfr_clear_flags ();
   inexact = mpfr_set_q (x, q, rnd);
+  flags = __gmpfr_flags;
 
   /* check values */
-  if (mpfr_cmp_str1(x, ys))
+  if (mpfr_cmp_str1 (x, ys))
     {
-      printf ("Error for q=%ld/%ld and rnd=%s\n", n, d,
+      printf ("Error for q = %ld/%ld and rnd = %s\n", n, d,
               mpfr_print_rnd_mode (rnd));
       printf ("correct result is %s, mpfr_set_q gives ", ys);
-      mpfr_out_str(stdout, 10, 0, x, MPFR_RNDN);
-      putchar('\n');
+      mpfr_out_str (stdout, 10, 0, x, MPFR_RNDN);
+      putchar ('\n');
       exit (1);
     }
 
@@ -56,12 +58,23 @@ check (long int n, long int d, mpfr_rnd_t rnd, const char *ys)
       exit (1);
     }
   compare = mpfr_cmp_si (t, n);
-  if (((inexact == 0) && (compare != 0)) ||
-      ((inexact < 0) && (compare >= 0)) ||
-      ((inexact > 0) && (compare <= 0)))
+  if (! SAME_SIGN (inexact, compare))
     {
-      printf ("wrong inexact flag: expected %d, got %d\n", compare,
-              inexact);
+      printf ("Wrong ternary value for q = %ld/%ld and rnd = %s:\n"
+              "expected %d or equivalent, got %d\n",
+              n, d, mpfr_print_rnd_mode (rnd), compare, inexact);
+      exit (1);
+    }
+
+  ex_flags = compare == 0 ? 0 : MPFR_FLAGS_INEXACT;
+  if (flags != ex_flags)
+    {
+      printf ("Wrong flags for q = %ld/%ld and rnd = %s:\n",
+              n, d, mpfr_print_rnd_mode (rnd));
+      printf ("Expected flags:");
+      flags_out (ex_flags);
+      printf ("Got flags:     ");
+      flags_out (flags);
       exit (1);
     }
 
@@ -84,8 +97,10 @@ check0 (void)
   mpq_set_si (y, 0, 1);
   for (r = 0; r < MPFR_RND_MAX; r++)
     {
-      inexact = mpfr_set_q(x, y, (mpfr_rnd_t) r);
-      if (!MPFR_IS_ZERO(x) || !MPFR_IS_POS(x) || inexact)
+      mpfr_clear_flags ();
+      inexact = mpfr_set_q (x, y, (mpfr_rnd_t) r);
+      if (!MPFR_IS_ZERO(x) || !MPFR_IS_POS(x) || inexact ||
+          __gmpfr_flags != 0)
         {
           printf("mpfr_set_q(x,0) failed for %s\n",
                  mpfr_print_rnd_mode ((mpfr_rnd_t) r));
@@ -115,11 +130,11 @@ check_nan_inf_mpq (void)
       mpfr_init2 (mpfr_cmp, MPFR_PREC_MIN);
       mpfr_set_nan (mpfr_cmp);
       printf ("mpfr_set_q with a NAN mpq value returned a wrong value :\n"
-              " waiting for ");
-      mpfr_print_binary (mpfr_cmp);
-      printf (" got ");
-      mpfr_print_binary (mpfr_value);
-      printf ("\n trinary value is %d\n", status);
+              "  expected ");
+      mpfr_dump (mpfr_cmp);
+      printf ("  got      ");
+      mpfr_dump (mpfr_value);
+      printf ("  ternary value is %d\n", status);
       exit (1);
     }
 
@@ -134,11 +149,11 @@ check_nan_inf_mpq (void)
       mpfr_init2 (mpfr_cmp, MPFR_PREC_MIN);
       mpfr_set_inf (mpfr_cmp, -1);
       printf ("mpfr_set_q with a -INF mpq value returned a wrong value :\n"
-              " waiting for ");
-      mpfr_print_binary (mpfr_cmp);
-      printf (" got ");
-      mpfr_print_binary (mpfr_value);
-      printf ("\n trinary value is %d\n", status);
+              "  expected ");
+      mpfr_dump (mpfr_cmp);
+      printf ("  got      ");
+      mpfr_dump (mpfr_value);
+      printf ("  ternary value is %d\n", status);
       exit (1);
     }
 
@@ -168,3 +183,13 @@ main (void)
   tests_end_mpfr ();
   return 0;
 }
+
+#else
+
+int
+main (void)
+{
+  return 77;
+}
+
+#endif
