@@ -1,6 +1,6 @@
 /* Test file for mpfr_set_str.
 
-Copyright 2004-2016 Free Software Foundation, Inc.
+Copyright 2004-2022 Free Software Foundation, Inc.
 Contributed by the AriC and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
@@ -17,13 +17,17 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
+https://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
-#include <stdlib.h>
-
 #include "mpfr-test.h"
 
+/* The implicit \0 is useless, but we do not write num_to_text[62] otherwise
+   g++ complains. */
+static const char num_to_text36[] = "0123456789abcdefghijklmnopqrstuvwxyz";
+static const char num_to_text62[] = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"
+  "abcdefghijklmnopqrstuvwxyz";
+
 static void
 check_special (void)
 {
@@ -900,6 +904,20 @@ check_overflow (void)
     {
       printf ("Check overflow failed (2) with:\n s='%s'\n x=", s);
       mpfr_dump (x);
+#if defined(__GNUC__)
+      printf ("This failure is triggered by GCC bug 86554:\n"
+              "  https://gcc.gnu.org/bugzilla/show_bug.cgi?id=86554\n"
+              "  https://gcc.gnu.org/bugzilla/show_bug.cgi?id=87276 "
+              "(about this test)\nWorkaround: disable code hoisting "
+              "with -fno-code-hoisting in CFLAGS.\n");
+      /* Note: In Debian, this error is obtained with gcc-snapshot from
+         20180908-1 to 20181127-1. With gcc-snapshot from 20181209-1 to
+         20190102-1 (at least), the MPFR build no longer seems affected
+         in general, but using --with-gmp-build=... together with
+         --enable-assert still triggers this failure. This bug has been
+         fixed in the GCC trunk rev 267725, thus the future gcc-snapshot
+         versions should no longer have this bug. */
+#endif
       exit (1);
     }
   mpfr_strtofr (x, "123456789E170141183460469231731687303715884105728",
@@ -940,6 +958,20 @@ check_overflow (void)
       mpfr_dump (x);
       exit (1);
     }
+  mpfr_strtofr (x, "1@2305843009213693951", &s, 16, MPFR_RNDN);
+  if (s[0] != 0 || !MPFR_IS_INF (x) || !MPFR_IS_POS (x))
+    {
+      printf ("Check overflow failed (8) with:\n s=%s\n x=", s);
+      mpfr_dump (x);
+      exit (1);
+    }
+  mpfr_strtofr (x, "1@2305843009213693951", &s, 17, MPFR_RNDN);
+  if (s[0] != 0 || !MPFR_IS_INF (x) || !MPFR_IS_POS (x))
+    {
+      printf ("Check overflow failed (9) with:\n s=%s\n x=", s);
+      mpfr_dump (x);
+      exit (1);
+    }
 
   /* Check underflow */
   mpfr_strtofr (x, "123456789E-2147483646", &s, 0, MPFR_RNDN);
@@ -971,6 +1003,13 @@ check_overflow (void)
       mpfr_dump (x);
       exit (1);
     }
+  mpfr_strtofr (x, "1@-2305843009213693952", &s, 16, MPFR_RNDN);
+  if (s[0] != 0 || !MPFR_IS_ZERO (x) || !MPFR_IS_POS (x) )
+    {
+      printf ("Check underflow failed (8) with:\n s='%s'\n x=", s);
+      mpfr_dump (x);
+      exit (1);
+    }
 
   mpfr_clear (x);
 }
@@ -1079,7 +1118,7 @@ bug20081028 (void)
 }
 
 /* check that 1.23e is correctly parsed, cf
-   http://gmplib.org/list-archives/gmp-bugs/2010-March/001898.html */
+   https://gmplib.org/list-archives/gmp-bugs/2010-March/001898.html */
 static void
 test20100310 (void)
 {
@@ -1159,14 +1198,14 @@ bug20120829 (void)
       s[4+i] = 0;
       inex1 = mpfr_mul_ui (e, e, 10, MPFR_RNDN);
       MPFR_ASSERTN (inex1 == 0);
-      RND_LOOP(r)
+      RND_LOOP_NO_RNDF (r)
         {
           mpfr_rnd_t rnd = (mpfr_rnd_t) r;
 
           inex1 = mpfr_exp10 (x1, e, rnd);
-          inex1 = SIGN (inex1);
+          inex1 = VSIGN (inex1);
           inex2 = mpfr_strtofr (x2, s, NULL, 0, rnd);
-          inex2 = SIGN (inex2);
+          inex2 = VSIGN (inex2);
           /* On 32-bit machines, for i = 7, r8389, r8391 and r8394 do:
              strtofr.c:...: MPFR assertion failed: cy == 0
              r8396 is OK.
@@ -1191,11 +1230,282 @@ bug20120829 (void)
   mpfr_clears (e, x1, x2, (mpfr_ptr) 0);
 }
 
+/* https://sympa.inria.fr/sympa/arc/mpfr/2016-12/msg00043.html
+   mpfr_strtofr can return an incorrect ternary value.
+   Note: As a consequence, the value can also be incorrect if the current
+   exponent range is not the maximum one (since the ternary value is used
+   to resolve double rounding in mpfr_check_range); this can happen only
+   if the value is a midpoint between 0 and the minimum positive number
+   or the opposite. */
+static void
+bug20161217 (void)
+{
+  mpfr_t fp, z;
+  static const char * num = "0.1387778780781445675529539585113525390625e31";
+  /* The above number is 5^47/2^9. */
+  int inex;
+
+  mpfr_init2 (fp, 110);
+  mpfr_init2 (z, 110);
+
+  inex = mpfr_strtofr (fp, num, NULL, 10, MPFR_RNDN);
+  MPFR_ASSERTN(inex == 0);
+  mpfr_set_str_binary (z, "10001100001000010011110110011101101001010000001011011110010001010100010100100110111101000010001011001100001101E-9");
+  MPFR_ASSERTN(mpfr_equal_p (fp, z));
+
+  /* try with 109 bits */
+  mpfr_set_prec (fp, 109);
+  inex = mpfr_strtofr (fp, num, NULL, 10, MPFR_RNDN);
+  MPFR_ASSERTN(inex < 0);
+  mpfr_set_str_binary (z, "10001100001000010011110110011101101001010000001011011110010001010100010100100110111101000010001011001100001100E-9");
+  MPFR_ASSERTN(mpfr_equal_p (fp, z));
+
+  mpfr_clear (fp);
+  mpfr_clear (z);
+}
+
+/* check bug in MPFR 3.1.5 is fixed: cf
+   https://sympa.inria.fr/sympa/arc/mpfr/2017-03/msg00009.html
+   Note: same bug as bug20161217. See also the comments of bug20161217;
+   here, this is a case where the value is incorrect. */
+static void
+bug20170308 (void)
+{
+  mpfr_exp_t emin;
+   /* the following is slightly larger than 2^-1075, thus should be rounded
+      to 0.5*2^-1074, with ternary value < 0 */
+  char str[] = "2.47032822920623272089E-324";
+  mpfr_t z;
+  int inex;
+
+  emin = mpfr_get_emin ();
+  mpfr_init2 (z, 53);
+  mpfr_set_emin (-1073);
+  /* with emin = -1073, the smallest positive number is 0.5*2^emin = 2^(-1074),
+     thus str should be rounded to 2^(-1074) with inex > 0 */
+  inex = mpfr_strtofr (z, str, NULL, 10, MPFR_RNDN);
+  MPFR_ASSERTN(inex > 0 && mpfr_cmp_ui_2exp (z, 1, -1074) == 0);
+  mpfr_set_emin (-1074);
+  /* with emin = -1074, str should be rounded to 2^(-1075) with inex < 0 */
+  inex = mpfr_strtofr (z, str, NULL, 10, MPFR_RNDN);
+  MPFR_ASSERTN(inex < 0 && mpfr_cmp_ui_2exp (z, 1, -1075) == 0);
+  mpfr_clear (z);
+  mpfr_set_emin (emin);
+}
+
+/* r13299 fails with 8-bit limbs (micro-gmp/8). */
+static void
+bug20181127 (void)
+{
+  char s[] = "9.Z6nrLVSMG1bDcCF2ONJdX@-183295525";  /* base 58 */
+  mpfr_t x, y;
+
+  mpfr_inits2 (6, x, y, (mpfr_ptr) 0);
+  mpfr_set_ui_2exp (x, 5, -1073741701, MPFR_RNDN);
+  mpfr_strtofr (y, s, NULL, 58, MPFR_RNDZ);
+  if (! mpfr_equal_p (x, y))
+    {
+      printf ("Error in bug20181127 on %s (base 58)\n", s);
+      printf ("Expected x = ");
+      mpfr_dump (x);
+      printf ("Got      y = ");
+      mpfr_dump (y);
+      printf ("*Note* In base 58, x ~= ");
+      mpfr_out_str (stdout, 58, 8, x, MPFR_RNDN);
+      printf ("\n");
+      exit (1);
+    }
+  mpfr_clears (x, y, (mpfr_ptr) 0);
+}
+
+static void
+coverage (void)
+{
+#if _MPFR_EXP_FORMAT >= 3 && _MPFR_PREC_FORMAT == 3 && MPFR_PREC_BITS == 64
+  char str3[] = "1@-2112009130072406892";
+  char str31[] = "1@-511170973314085831";
+  mpfr_t x;
+  int inex;
+  mpfr_exp_t emin;
+
+  /* exercise assertion cy == 0 around line 698 of strtofr.c */
+  emin = mpfr_get_emin ();
+  mpfr_set_emin (mpfr_get_emin_min ());
+  /* emin = -4611686018427387903 on a 64-bit machine */
+  mpfr_init2 (x, 1);
+  inex = mpfr_strtofr (x, str3, NULL, 3, MPFR_RNDN);
+  /* 3^-2112009130072406892 is slightly larger than (2^64)^-52303988630398057
+     thus we should get inex < 0 */
+  MPFR_ASSERTN(inex < 0);
+  MPFR_ASSERTN(mpfr_cmp_ui_2exp (x, 1, -52303988630398057 * 64) == 0);
+  inex = mpfr_strtofr (x, str31, NULL, 31, MPFR_RNDN);
+  /* 31^-511170973314085831 is slightly smaller than (2^64)^-39569396093273623
+     thus we should get inex > 0 */
+  MPFR_ASSERTN(inex > 0);
+  MPFR_ASSERTN(mpfr_cmp_ui_2exp (x, 1, -39569396093273623 * 64) == 0);
+  mpfr_clear (x);
+  mpfr_set_emin (emin);
+#endif
+}
+
+#define BSIZE 512
+
+static void
+random_tests (void)
+{
+  char s0[BSIZE], s1[BSIZE], s2[BSIZE+64];
+  mpfr_t x0, x1, x2;
+  int prec, i;
+
+  for (prec = MPFR_PREC_MIN; prec < 300; prec++)
+    {
+      mpfr_inits2 (prec, x0, x1, x2, (mpfr_ptr) 0);
+
+      for (i = 0; i < 20; i++)
+        {
+          const char *num_to_text;
+          mpfr_exp_t e0, e1;
+          int base, j, k, neg;
+          int noteq = 0;
+          char d;
+
+          /* We want the same exponent for x0 and its successor x1.
+             This is not possible for precision 1 in base 2. */
+          do
+            base = 2 + (randlimb () % 61);
+          while (prec == 1 && base == 2);
+
+          num_to_text = base <= 36 ? num_to_text36 : num_to_text62;
+
+          do
+            {
+              /* Let's consider only positive numbers. We should test
+                 negative numbers, but they should be built later, just
+                 for the test itself. */
+              tests_default_random (x0, 0,
+                                    mpfr_get_emin (), mpfr_get_emax (), 1);
+              mpfr_set (x1, x0, MPFR_RNDN);
+              mpfr_nextabove (x1);
+              mpfr_get_str (s0, &e0, base, BSIZE - 1, x0, MPFR_RNDU);
+              mpfr_get_str (s1, &e1, base, BSIZE - 1, x1, MPFR_RNDD);
+            }
+          while (! (mpfr_regular_p (x0) && mpfr_regular_p (x1) && e0 == e1));
+
+          /* 0 < x0 <= (s0,e) <= (s1,e) <= x1 with e = e0 = e1.
+             Let's build a string s2 randomly formed by:
+             - the common prefix of s0 and s1;
+             - some of the following digits of s0 (possibly none);
+             - the next digit of s0 + 1;
+             - some of the following digits of s1 (possibly none).
+             Then 0 < x0 <= (s0,e) < (s2,e) <= (s1,e) <= x1, and with
+             a very high probability that (s2,e) < (s1,e); noteq is
+             set to true in this case.
+             For instance, if:
+               s0 = 123456789
+               s1 = 124012345
+             one can have, e.g.:
+               s2 = 12345734
+               s2 = 123556789
+               s2 = 124
+               s2 = 124012
+             s2 is not taken completely randomly between s0 and s1, but it
+             will be built rather easily, and with the randomness of x0,
+             we should cover all cases, with s2 very close to s0, s2 very
+             close to s1, or not too close to either. */
+
+          neg = randlimb () & 1;
+          s2[0] = neg ? '-' : '+';
+          s2[1] = '.';
+
+          for (j = 0;
+               MPFR_ASSERTN (s0[j] != 0 && s1[j] != 0), s0[j] == s1[j];
+               j++)
+            s2[j+2] = s0[j];
+
+          /* k is the position of the first differing digit. */
+          k = j;
+
+          while (j < BSIZE - 2 && randlimb () % 8 != 0)
+            {
+              MPFR_ASSERTN (s0[j] != 0);
+              s2[j+2] = s0[j];
+              j++;
+            }
+
+          MPFR_ASSERTN (s0[j] != 0);
+          /* We will increment the next digit. Thus while s0[j] is the
+             maximum digit, go back until this is no longer the case
+             (the first digit after the common prefix cannot be the
+             maximum digit, so that we will stop early enough). */
+          while ((d = s0[j]) == num_to_text[base - 1])
+            j--;
+          noteq = j != k;
+          s2[j+2] = d = *(strchr (num_to_text, d) + 1);
+          if (d != s1[j])
+            noteq = 1;
+          j++;
+
+          while (j < BSIZE - 1 && randlimb () % 8 != 0)
+            {
+              MPFR_ASSERTN (s1[j] != 0);
+              s2[j+2] = s1[j];
+              j++;
+            }
+
+          sprintf (s2 + (j+2), "@%" MPFR_EXP_FSPEC "d", (mpfr_eexp_t) e0);
+
+          while (noteq == 0 && j < BSIZE - 1)
+            {
+              if (s1[j] != '0')
+                noteq = 1;
+              j++;
+            }
+
+          if (neg)
+            {
+              mpfr_neg (x0, x0, MPFR_RNDN);
+              mpfr_neg (x1, x1, MPFR_RNDN);
+            }
+
+          if (noteq)
+            {
+              mpfr_strtofr (x2, s2, NULL, base, MPFR_RNDZ);
+              if (! mpfr_equal_p (x2, x0))
+                {
+                  printf ("Error in random_tests for prec=%d i=%d base=%d\n",
+                          prec, i, base);
+                  printf ("s0 = %s\ns1 = %s\ns2 = %s\n", s0, s1, s2);
+                  printf ("x0 = ");
+                  mpfr_dump (x0);
+                  printf ("x2 = ");
+                  mpfr_dump (x2);
+                  exit (1);
+                }
+            }
+
+          mpfr_strtofr (x2, s2, NULL, base, MPFR_RNDA);
+          if (! mpfr_equal_p (x2, x1))
+            {
+              printf ("Error in random_tests for prec=%d i=%d base=%d\n",
+                      prec, i, base);
+              printf ("s0 = %s\ns1 = %s\ns2 = %s\n", s0, s1, s2);
+              printf ("x1 = ");
+              mpfr_dump (x1);
+              printf ("x2 = ");
+              mpfr_dump (x2);
+              exit (1);
+            }
+        }
+      mpfr_clears (x0, x1, x2, (mpfr_ptr) 0);
+    }
+}
+
 int
 main (int argc, char *argv[])
 {
   tests_start_mpfr ();
 
+  coverage ();
   check_special();
   check_reftable ();
   check_parse ();
@@ -1205,6 +1515,10 @@ main (int argc, char *argv[])
   test20100310 ();
   bug20120814 ();
   bug20120829 ();
+  bug20161217 ();
+  bug20170308 ();
+  bug20181127 ();
+  random_tests ();
 
   tests_end_mpfr ();
   return 0;
