@@ -1,6 +1,6 @@
 /* Test file for mpfr_tanh.
 
-Copyright 2001-2018 Free Software Foundation, Inc.
+Copyright 2001-2022 Free Software Foundation, Inc.
 Contributed by the AriC and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
@@ -17,7 +17,7 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
+https://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
 #include "mpfr-test.h"
@@ -118,7 +118,7 @@ special_overflow (void)
 
 /* This test was generated from bad_cases, with input y=-7.778@-1 = -3823/8192.
    For the x value below, we have atanh(y) < x, thus since tanh() is increasing,
-   y < tanh(x), and thus tanh(x) rounded towards zero should give -3822/8192. */
+   y < tanh(x), and thus tanh(x) rounded toward zero should give -3822/8192. */
 static void
 bug20171218 (void)
 {
